#!/usr/bin/env python3

import os
import subprocess
import filecmp

class ApBootstrapMachineID(object):
    def __init__(self, machine_id_dbus_file: str, machine_id_file: str, machine_root_directory: str):

        # Variables passed in during initialization
        self.machine_id_dbus_file = machine_id_dbus_file
        self.machine_id_file = machine_id_file
        self.machine_root_directory = machine_root_directory
        self.dbus_uuigen = "/usr/bin/dbus-uuidgen --ensure=" + self.machine_id_dbus_file
        self.systemd_machine_id_setup = "/bin/systemd-machine-id-setup --root=" + self.machine_root_directory

    # Delete the dbus and systemd assigned machine-id files
    def delete_old_machine_ids(self):
        try:
            os.remove(self.machine_id_dbus_file)
        except OSError:
            pass

        try:
            os.remove(self.machine_id_file)
        except OSError:
            pass

    # Generate a fresh dbus machine-id file
    def generate_dbus_machine_id(self) -> bool:
        command = self.dbus_uuigen

        try:
            subprocess.run(command, timeout=10, shell=True)
        except subprocess.TimeoutExpired:
            return False
        except subprocess.CalledProcessError:
            return False

        if not (os.path.isfile(self.machine_id_dbus_file)):
            return False

        return True

    # Generate a fresh systemd machine-id based on the dbus machine-id
    def generate_systemd_machine_id(self) -> bool:
        command = self.systemd_machine_id_setup

        try:
            subprocess.run(command, timeout=10, shell=True)
        except subprocess.TimeoutExpired:
            return False
        except subprocess.CalledProcessError:
            return False

        if not (os.path.isfile(self.machine_id_file)):
            return False

        return True

    # Make sure the machine-id files match
    def compare_machine_id_files(self) -> bool:
        if not filecmp.cmp(self.machine_id_dbus_file, self.machine_id_file, shallow=False):
            return False

        return True

    # Function that walks through the steps necessary to configure machine_id by calling other functions
    def configure_machine_id(self) -> bool:
        # Delete Old machine ID's, nothing to check for since we try to delete OS file and pass if failure
        self.delete_old_machine_ids()

        if not (self.generate_dbus_machine_id()):
            print("apbootstrap_configure_machine_id: Could not generate dbus machine id (apbootstrap_configure_machine_id.generate_dbus_machine_id)")
            return False

        if not (self.generate_systemd_machine_id()):
            print("apbootstrap_configure_machine_id: Could not generate systemd machine id (apbootstrap_configure_machine_id.generate_systemd_machine_id)")
            return False

        if not (self.compare_machine_id_files()):
            print("apbootstrap_configure_machine_id: machine id files did not match; comparison failed (apbootstrap_configure_machine_id.compare_machine_id_files)")
            return False

        return True

